package project.model;

public class CanopyLayer extends Object
{
   private int hoursOfSun[][][]; // first dimension is for 14 hours and 1 total index.  second dimension is for x, third for y
   /** This function initiaates the grid to have either 14 value for the total sun index (14), or no shadow for the shadow/hour indices (0-13)
    @param layers this is an array of grids from the model, that it uses to calculate sun coverage
    */
   private void startGrid(Grid[] layers)
   {   hoursOfSun = new int[Constants.HOURS_OF_SUN + 1][layers[0].getWidth()][layers[0].getHeight()];
       for(int x = 0;x < hoursOfSun[0].length;x++)
       {   for(int y =0;y < hoursOfSun[0][0].length;y++)
           {   hoursOfSun[Constants.TOTAL_SUN_HOURS_INDEX][x][y] = Constants.HOURS_OF_SUN; // the hoursofsun data must be set to 14 by default, as there are no shadows
               for(int hours = 0;hours < Constants.HOURS_OF_SUN-1;hours++)
               {   hoursOfSun[hours][x][y] = 0; // the shading data must be set to zero, as there is no shade by default
               }
           }
       }
   }
   /** This returns the hours of sun at a given point.  Not to be confused with the shade array stuff
    @return the number of hours of sun at a given xy coordinate
    */
   public int getHoursOfSun(int x,int y)
   {   return this.hoursOfSun[Constants.TOTAL_SUN_HOURS_INDEX][x][y];
   }
   /** This constructs the canopy layer and calculates all the shadowing
    @param layers an array of grid objects that contain data on shading
    */
   public CanopyLayer(Grid[] layers)
   {    super();
        this.startGrid(layers); // initiates the grid to default values
        for(int shift = -3;shift <= +3;shift++) // squares started shifted 3 squares left, and they end 3 squares right
        {   for(int x= 0;x < layers[Constants.BASE_LAYER].getWidth();x++) // x goes from 0 to width
            {   for(int y = 0;y < layers[Constants.BASE_LAYER].getHeight();y++) // y goes from 0 to height
                {   for(int layer = 0;layer < 2;layer++) // we go through both layers
                    {   if(layers[layer].getShadowSize(x,y) != -1) // if we have a valid shadow size (default is -1, so anything but that is a shadow-caster)
                        {    castShadow(layers[layer].getShadowSize(x,y),shift,x + shift,y); // cast the shadow at the current x/y coordinate
                        }
                    }
                }

            }
        }
    }
    /** Based on a given xy point for a tree (for instance), this takes it's size, and what hour we're on (the shift), and shades the appropriate region
     @param size - the size of the shadow cast by this tree. 0 = itself, 1 = 8 squares around it, 2 = 24 squares around it
     @param shift - the current shift (Based on angle of sun) of the shadow.  At noon shift = 0;
     @param cx - the x coordinate of the shadow-casting object
     @param cy - the y coordinate of the shadow-casting object
     */
    private void castShadow(int size,int shift,int cx,int cy)
    {
        for(int x = cx - size;x <= cx + size;x++)  // goes from the (origin-shadowSize) to (origin+shadowSize)
        {   for(int y = cy - size;y <= cy + size;y++)
            {
                if(coordsValid(x,y) && hoursOfSun[Constants.TOTAL_SUN_HOURS_INDEX][x][y] >= 2 && (x != (cx - shift) || y != cy)) // if our coordinates are valid, and there are still hours of sunlight to subtract...
                {   if(hoursOfSun[((shift + 4) * 2)-2][x][y] == 0) // if there is no shade here yet, then
                    {   hoursOfSun[Constants.TOTAL_SUN_HOURS_INDEX][x][y] -= 2; // subtract from the sunlight hours

                    }
                    hoursOfSun[((shift + 4) * 2)-1][x][y] = 1; // tell the array that there is now shade here
                    hoursOfSun[((shift + 4) * 2)-2][x][y] = 1; // tell the array that there is now shade at the hour before (since hours move in multiples of 2)
                    //Debug.O("casting shadow.  shift = " + shift + " size = " + size + " point = " + cx + ", " + cy);
                }
            }
        }
    }
    
    /** returns an int representing if there is a shadow at a given hour at a given coordinate
     @param x - the x coordinate desired
     @param y - the y coordinate desired
     @param hour the hour desired
     @return a 0 (no shade) or a 1 (shade)
     */
    public int getShadowAtHour(int x,int y,int hour)
    {   return this.hoursOfSun[hour][x][y];
    }
    /** returns if the x/y coordinates generated in the castShadow function are valid.  Since it may end up negative, this is a imperative check
     @param x - the x coordinate
     @param y - the y coordinate
     @return a true or false value saying if the coordinate is valid
     */
    private boolean coordsValid(int x,int y)
    {
        boolean xNotZero = x >= 0;
        boolean xNotBig = x < this.hoursOfSun[0].length;
        boolean yNotZero = y >= 0;
        boolean yNotBig = y < this.hoursOfSun[0][0].length;
        boolean isValid = xNotZero && xNotBig && yNotZero && yNotBig;
        Debug.O(x + ", " + y + ": " + isValid);
        return isValid;
    }

}