package project.model;
import becker.io.*;
import java.awt.Polygon;
public class Grid extends Object
{
   private Square[][] squares;
   private SquareList mySL;
   private LandscapePlanner model;
   /** This constructs the grid with empty squares
    @param height the height of the grid
    @param width the width of the grid
    */
   public Grid(int height, int width)
   {  squares = new Square[height][width];

      for(int x = 0;x < width;x++)
      {   for(int y = 0;y < height;y++)
          {  squares[x][y] = new Square();
          }
      }
   }
   /** This returns the shadow size of a given squareContents in the grid
    @param x,y the coordinates
    @return the shadow size.  This is either -1, 0, 1, or 2
    */
   public int getShadowSize(int x,int y)
   {   return this.squares[x][y].getContents().getShadowSize ();
   }
   /** This sets topsoil (true or false) in a given rectangle
    @param where the location we want the topsoil to go
    @param ts whether we're setting or removing topsoil
    */
   public void setTopsoil(Rect where,boolean ts)
   {  for(int x = where.getLeft();x < where.getRight();x++)
      {  for(int y = where.getTop();y < where.getBottom();y++)
         {   this.squares[x][y].setTopsoil (ts);
         }
      }
   }
   /** This gets the maintenance cost of this grid
    @return the total maintanence cost of the grid
    */
   public int getMaintainCost()
   {  int cost = 0;
      for(int x = 0;x < squares.length;x++)
      {   for(int y = 0;y < squares[0].length;y++)
          {   cost += this.squares[x][y].getMaintainCost(mySL);
          }
      }
      return cost;
   }
   /** This gets the install cost of the grid
    @return the installation cost of the entire grid
    */
   public int getInstallCost()
   {  int cost = 0;
      for(int x = 0;x < squares.length;x++)
      {   for(int y = 0;y < squares[0].length;y++)
          {   cost += this.squares[x][y].getInstallCost(mySL);
          }
      }
      return cost;
   }
   /** This is essentially the same as the Grid(int,int) constructor, except it's different signature makes it construct slightly differently
   @param height, width - well gee-whiz, I wonder what these do
   @param isPlants - this just modifies the signature of the constructor and related constructors, it isn't actually used
   */
   public Grid(int height, int width, boolean isPlants)
   {  squares = new Square[height][width];

      for(int x = 0;x < width;x++)
      {   for(int y = 0;y < height;y++)
          {  squares[x][y] = new Square(isPlants);
          }
      }
   }
   /** This fills up a polygon with withWhat squareContents
   @param withWhat what are we filling
   @param myGon where are we filling it
   */
   public void fillGon(SquareContents withWhat,Polygon myGon)
   {  for(int x =0;x < squares.length;x++)
      {   for(int y =0;y < squares[0].length;y++)
          {   if(myGon.contains(x,y) && squares[x][y].isBlank())
              {    this.setSquare(x,y,withWhat);
              }
          }
      }
   }
   /** This returns if this grid is full.  It is used mainly in the base grid
   @return true or false if the grid is full or not
   */
   public boolean isFull()
   {  for(int x = 0;x < squares.length;x++)
      {   for(int y = 0;y < squares[0].length;y++)
          {   if (squares[x][y].isBlank())
              {  return false;
              }
          }
      }
      return true;
   }
   /** This creates a new grid based on height, width, a textInput, and the all-important squareList
    @param height, width - hur hur, they are the height and width of the square
    @param in - the textInput object initialized to the file we're reading from
    @param squareList - a squareList object to aid in the square's building
    */
   public Grid(int height, int width, TextInput in, SquareList squareList)
   {  squares = new Square[height][width];
      mySL = squareList;

      for(int x = 0;x < width;x++)
      {   for(int y = 0;y < height;y++)
          {  squares[y][x] = new Square(in, squareList);
          }
          in.readLine();
      }

   }
   /** This saves to a given (initialized) textOutput file
    @param out this is a textOutput where we can throw data at and have a saved game
    */
   public void save(TextOutput out)
   {  for(int x = 0;x < squares.length;x++)
      {   for(int y =0;y< squares[0].length;y++)
          {   squares[y][x].save(out);
          }
          out.println("");
      }
   }
   /** This is another function to save things.  Since we can't save mulch and topsoil at the same time as the square data, we need a new function
   @param out - a textoutput initialized to the file we want to output to
   */
   public void saveMulch(TextOutput out)
   {   for(int x = 0;x < squares.length;x++)
       {   for(int y = 0;y < squares[0].length;y++)
           {   squares[y][x].saveMulch(out);
           }
           out.println("");
       }
   }
   /** This copies another grid onto this grid.  This is used in the saveGrids method in the model
    @param source - the source grid to be copied
    */
   public Grid(Grid source)
   {  if(source != null)
      {   this.squares = new Square[source.getWidth()][source.getHeight()];
          for(int x = 0;x < source.getWidth();x++)
          {   for(int y = 0;y < source.getHeight();y++)
              {   this.squares[x][y] = new Square(source.getSquare(x,y));
              }
          }
      }
      this.mySL = source.getSL();
      this.model = source.getModel();
   }
   /** returns this thing's width */
   public int getWidth()
   {   return squares.length;
   }
   /** returns this thing's height */
   public int getHeight()
   {   return squares[0].length;
   }
   /** returns this thing's squareList */
   public SquareList getSL()
   {   return this.mySL;
   }
   /** returns this thing's model */
   public LandscapePlanner getModel()
   {   return this.model;
   }
   /** loads the topsoil and mulch when we're loading stuff, since they can't be done at the same time
   @param in the initialized textInput currently at the soil/mulch area of the file
   @param width, height - the width and height of this grid
   */
   public void loadSoilMulch(TextInput in,int width,int height)
   {  for(int x =0;x < width;x++)
      {   for(int y= 0;y < height;y++)
          {   squares[y][x].loadMulch(in);
          }
          in.readLine();
      }
   }
   /** returns the square data from a given x-y coordinate */
   public Square getSquare(int x, int y)
   {  return squares[x][y];
   }
   /** this sets the 8 square radius around a given x-y coordinate to have water
    @param x,y the point that has water.
    @param ibw whether or not there is water here.
    */
   public void setWater(int x,int y,boolean ibw)
   {  int left = x - 1;
      int right = x + 1;
      int top = y - 1;
      int bottom = y + 1;
      if(left < 0) {left = 0;};
      if(right >= Constants.DEFAULT_WIDTH) {right = Constants.DEFAULT_WIDTH-1;};
      if(bottom >= Constants.DEFAULT_HEIGHT) {bottom = Constants.DEFAULT_HEIGHT-1;};
      if(top < 0) {top = 0;};

      for( x = left;x <= right;x++)
      {  for( y = top;y <= bottom;y++)
         {   this.squares[x][y].setBesideWater(ibw);
             //Debug.O("setting (" + x +", " +y + " to have water");
         }
      }
   }
   /** This adds or removes mulch (depending on m) in a given rectangle
    @param the rectangle where the mulch is desired
    @param m whether we're adding (true) or removing (false)
    */
   public void setMulch(Rect where,boolean m)
   {  for(int x = where.getLeft();x < where.getRight();x++)
      {  for(int y = where.getTop();y < where.getBottom();y++)
         {   this.squares[x][y].setMulch(m);
         }
      }
   }
   /** This clears out all the water in the area.  This was the only easy way we figured out how to accurately replot the water coverage when some was removed */
   public void clearWater()
   {  for(int x = 0;x < squares.length;x++)
      {  for(int y = 0;y < squares[0].length;y++)
         {   this.squares[x][y].setBesideWater(false);
         }
      }
   }
   /** this sets all the squares in a rectangle to get whatToPut squareContents, if they don't have something already
    @param where - the rectangle that is the target
    @param whatToPut - a reference to a squareContents in the squareList that is what we're going to put
    @param overWrite - if we're overwriting.  This is true when we're slapping down uneditable squares in the plant layer
    */
   public void setRect(Rect where,SquareContents whatToPut, boolean overwrite)
   {    for(int x = where.getLeft();x < where.getRight();x++)
        {   for(int y = where.getTop();y < where.getBottom();y++)
            {    if(squares[x][y].isBlank()||overwrite)
                 {   //Debug.O("Placing Stuff");
                     //Debug.O("" + whatToPut.getName());
                     if(((whatToPut instanceof OrganicContents) && goodSoil(whatToPut,x,y)) || !(whatToPut instanceof OrganicContents))
                     {  this.setSquare(x,y,whatToPut);
                     }
                 }
            }
        }
   }
   /** Whether or not there is good enough soil at the x-y coords.
    @return if whatToPut can tolerate the soil at the given square
    @param whatToPut what plant we're talking about
    @param x,y the x-y coordinates
    */
   private boolean goodSoil(SquareContents whatToPut, int x,int y)
   {  OrganicContents temp = (OrganicContents)whatToPut;
      if(squares[x][y].getTopsoil() || temp.getSoilReq() == 0 || temp.getSoilReq ()== 2)// if we have topsoil, or if the plant requires any or poor soil, then the soil is adequate
      {   Debug.O("needs soil type: " + temp.getSoilReq() + " getTopSoil: " + squares[x][y].getTopsoil());
          return true;
      }
      else
      {  return false;
      }
   }
   /** This removes a rectangle, and sets all the squares in it to be empty squares
    @param where - where we're doing the removing
    @param sl - used to get the empty square reference from the squarelist
    */
   public void removeRect(Rect where,SquareList sl)
   {  for(int x = where.getLeft();x < where.getRight();x++)
      {  for(int y = where.getTop();y < where.getBottom();y++)
         {  this.setSquare(x,y,sl.getSquare(Constants.EMPTY_SQUARE));
         }
      }
   }
   /** This removes a rectangle in the plant layer
    @param where - where we are removing stuff from
    @param sl - the squarelist so we can do comparisons about square types
    @param baseGrid - the base grid, so we can check for soil there before we remove stuff
    */
   public void removeRect(Rect where, SquareList sl,Grid baseGrid) // makes sure that soil exists on the base layer before removing in the plant layer
   {   for(int x = where.getLeft();x < where.getRight();x++)
       {  for(int y = where.getTop();y < where.getBottom();y++)
          {   if(baseGrid.getSquare(x,y).getContents().equals(sl.getSquare(Constants.NUM_SOIL)))
              {   this.setSquare(x,y,sl.getSquare(Constants.EMPTY_SQUARE));
              }
          }
       }
   }

   /** This is a basic method that places a whatToPut at the given x-y coordinates
    @param x,y - the x-y coords we want to put something at
    @param whatToPut - a reference to a squareContents object that we are planting
    */
   private void setSquare(int x,int y,SquareContents whatToPut)
   {
        squares[x][y].addContents(whatToPut);
   }


}
