package project.model;
import becker.io.*;
import java.awt.Color;

public class OrganicContents extends SquareContents
{
   private int sunReq;
   private int soilReq;
   private int buyCost;
   private int density;
   private int air;
   private int enviroBonus;
   private String[] enviroDesc;
   private int waterReq;
   private boolean isTree;
   private int height;
   private int age;
   private int installCost;
   private boolean isDecid; // is this a deciduous tree?
   private boolean blooms; // does this flower bloom?
   
   /** General constructor
   */
   public OrganicContents()
   {  super();
   }

   /** Constructor when loading from a file
   */
   public int getCost()
   {   return this.buyCost;
   }
   /** this helper method is called in the constructor, and determines the height from the description.
    @return the height of this OrganicContents object
    */
   private int findHeight()
   {   int whereMetres = this.description.indexOf("metres"); // where we find meters, we will find the height
       String numbersMaybe;
       if (whereMetres >= 3)
       {   numbersMaybe = this.description.substring(whereMetres -3,whereMetres);
       }
       else
       {   return 0;
       }
       Debug.O("numbersMaybe = " + numbersMaybe);
       int height = Integer.parseInt(numbersMaybe.trim());

       return height;
   }
   /** this method looks at the height of the plant and the type of the plant and determines if it casts a shadow
    @return the radius of the shadow from the base of the plant
   */
   private int findShadow()
   {  if(this.isDecid)
      {   if(this.height < 3)
          {   return 0;
          }
          else if(this.height < 8)
          {   return 1;
          }
          else
          {   return 2;
          }
      }
      else
      {   if(this.height < 7)
          {   return 0;
          }
          else if(this.height > 0)
          {   return 1;
          }
          else // if the height isn't more than zero, than this is a flower or something, and doesn't cast shadows
          {   return -1;
          }
      }
   }
   /** this accessor method returns if an organic contents is a deciduous tree or not
    @return whether or not this object is deciduous
    */
   public boolean getIsDecid()
   {   return this.isDecid;
   }
   /** this constructor constructs the organiccontents from a given textfile, and parses a lot from the description
    @param in a textinput initialized to the plantlist.txt file.
    */
   public OrganicContents(TextInput in)
   {  super(); // initializes the basic squareContents stuff
      this.fullName = in.readLine(); // gets fullname
      this.description = in.readLine(); // gets the description
      int red = Integer.parseInt (in.readLine()); // gets how red this should be
      int green = Integer.parseInt (in.readLine()); // gets how green this should be
      int blue = Integer.parseInt (in.readLine()); // gets how blue this should be
      this.myColor = new Color(red,green,blue); // creates the color from the information from above
      this.displayText = in.readLine(); // gets the display text
      this.sunReq = Integer.parseInt (in.readLine()); // gets the sunlight requirements.  0 = shade, 1 = partial, 2 = full, 3 = any
      this.soilReq = Integer.parseInt (in.readLine()); // gets the soil requirements.
      this.density = Integer.parseInt (in.readLine()); // gets the density of this plant
      this.waterReq = Integer.parseInt (in.readLine()); // gets the water requirements of this plant
      this.buyCost = Integer.parseInt (in.readLine()); // gets the purchase cost of this plant
      this.air = Integer.parseInt (in.readLine()); // gets the air production of this plant
      int numEnviro = Integer.parseInt (in.readLine()); // gets the number of environmental bonus factors this plant has
      this.enviroDesc = new String [numEnviro]; // initializes the environmental description array

      this.height = findHeight(); // figures out the height from the description

      in.readLine(); // I screwed up something in the program that generates plantlist.txt, so there are 2 blank lines here
      in.readLine();
      for(int x=0;x<numEnviro;x++) // goes through all the environmental data it found
      { this.enviroBonus += Integer.parseInt (in.readLine()); // gets the bonus (plus or minus) from the file
        this.enviroDesc[x] = in.readLine(); // gets the description of the bonus
      }
      String decid = "deciduous"; // here we determine if the plant is deciduous or not
      this.isDecid = (this.description.toUpperCase().indexOf(decid.toUpperCase()) != -1);


      String blooms = "bloom"; // here we determine if the plant blooms or not
      this.blooms = (this.description.toUpperCase().indexOf(blooms.toUpperCase()) != -1);

      // here we determine if the plant is a tree or not
      this.isTree = (this.description.toUpperCase().lastIndexOf("TREE") != -1) || (this.description.toUpperCase().lastIndexOf("SHRUB") != -1) || (this.description.toUpperCase().lastIndexOf("DECIDUOUS") != -1) || (this.description.toUpperCase().lastIndexOf("CONIFEROUS") != -1);
      this.shadowSize = findShadow();
   }
   /** this function returns if this plant is a tree
    @return if this is a tree
    */
   public boolean getIsTree()
   {  return this.isTree;
   }
   /** this function returns if this plant blooms or not
    @return if this thing blooms
    */
   public boolean getBlooms()
   {  return this.blooms;
   }
   /** this function returns if this plant is deciduous or not
    @return if this is deciduous
    */
   public boolean getDecid()
   {  return this.isDecid;
   }
   /** Returns the sunlight requirement
   @return sunlight requirement
   */
   public int getSunReq()
   {  return sunReq;
   }

   /** Returns the soil requirement
   @return soil requirement
   */
   public int getSoilReq()
   {  return soilReq;
   }

   /** Return the buying cost of the plant
   @return buying cost
   */
   public int getBuyCost()
   {  return buyCost;
   }

   /** Return the CO2 Consumption/O2 Production rate
   @return CO2/O2 Rate
   */
   public int getAir()
   {  return air;
   }

   /** Return the enviromental bonus of the plant
   @return enviromental bonus
   */
   public int getEnviroBonus()
   {  return enviroBonus;
   }

   /** Return the watering requirement of the plant
   @return watering requirement
   */
   public int getWaterReq()
   {  return waterReq;
   }

   /** Shows if the plant is a tree
   @return True if plant is a tree, False if plant is a flower
   */
   public boolean getIfTree()
   {  return isTree;
   }
   /** Returns if this plant has a problem with it's lighting conditions
    @param hoursOfSun the hours of sun that this plant recieves
    */
   public boolean lightProblem(int hoursOfSun)
   {  int hoursOfShade = 14-hoursOfSun;
      if(this.sunReq == 0)
      {   return hoursOfShade <= 6;
      }
      else if(this.sunReq == 1)
      {   return (hoursOfShade <= 2 || hoursOfShade >= 6);
      }
      else // (this.sunReq == 2), it's the only possibility left
      {   return (hoursOfShade >=2);
      }
   }
   /** Returns the height of the tree
   @return height
   */
   public int getHeight()
   {  if (this.getIfTree())
      { return height;
      }
      else
      { return 0;
      }
   }

   /** Return the installation cost of the plant
   @return installation cost
   */
   public int getInstallCost()
   {  return installCost;
   }

   /** Return the age of the plant
   @return age
   */
   public int getAge()
   {  return age;
   }

   /** Return the type of the plant
   @return plant type
   */
   public int getType()
   {  return myType;
   }

}