package project.model;

public class Rect extends Object
{
    private int left;
    private int top;
    private int width;
    private int height;
    private int right;
    private int bottom;
    /** this constructor constructs the rect with 4 parameters, and determines the other 2 it needs
     @param left - the leftmost x coordinate
     @param top - the topmost y coordinate
     @param width - the width of the rectangle
     @param height - the height of the rectangle
     */
    public Rect(int left, int top, int width, int height)
    {   this.left = left;
        this.top = top;
        this.width = width;
        this.height = height;
        this.right = this.left + this.width;
        this.bottom = this.top + this.height;
    }
    /** this method determines if the rectangle contains a given x-y point
    @param x,y the point we are trying to determine this rectangle contains
    @return whether or not the rectangle contains the given point
    */
    public boolean contains(int x,int y)
    {   return (x>= this.left && x <= this.right && y >= this.top && y <= this.bottom);

    }
    /** this constructor constructs the rectangle to have left, top, bottom, right, to be the left and top values passed
     @param left the x coordinate of this tiny rectangle
     @param top the y coordinate of this tiny rectangle
     */
    public Rect(int left, int top)
    {   this.left = left;
        this.top = top;
        this.width = 0;
        this.height = 0;
        this.right = left;
        this.bottom = top;
    }
    /** this method sets the other corner of the rectangle.  It is called from LandscapeModel::UpdateData;
     @param right the right corner to be set
     @param bottom the bottom corner to be set
     */
    public void setCorner(int right, int bottom)
    {   this.right = right;
        this.bottom = bottom;
        this.width = Math.abs(right - left);
        this.height = Math.abs(bottom - top);
    }
    /** this method returns the left (leftmost x) coordinate of this rectangle
     @return the leftmost x coordinate
     */
    public int getLeft()
    {   return this.left;
    }
    /** this method returns the top (topmost y) coordinate of this rectangle
     @return the topmost y coordinate
     */
    public int getTop()
    {   return this.top;
    }
    /** this method returns the width of this rectangle
     @return the width of the rectangle
     */
    public int getWidth()
    {   return this.width;
    }
    /** this method returns the height of this rectangle
     @return the height of the rectangle
     */
    public int getHeight()
    {   return this.height;
    }
    /** this method returns the rightmost x coordinate of this rectangle
     @return the rightmost x coordinate
     */
    public int getRight()
    {   return this.right;
    }
    /** this method returns the bottom most y coordinate of this rectangle
     @return the bottom y coordinate
     */
    public int getBottom()
    {   return this.bottom;
    }
    /** this method sets the left coordinate of the rectangle, and makes sure that it jives with the current right-side coordinate
     @param left the new left coordinate of this rectangle
     */
    public void setLeft(int left)
    {   this.left = left;
        if(this.left > this.right)
        {   int temp = this.left;
            this.left = this.right;
            this.right = temp;
        }
    }
    /** this method sets the topmost coordinate, and makes sure it jives with the bottom coordinate
     @param top the new top coordinate of the rectangle
     */
    public void setTop(int top)
    {   this.top = top;
        if(this.top > this.bottom)
        {  int temp = this.top;
           this.top = this.bottom;
           this.bottom  =temp;
        }
    }
    /** this method sets the bottom coordinate of the rectangle, and makes sure it jives with the top coordinate
     @param bottom the new bottom coordinate
     */
    public void setBottom(int bottom)
    {   this.bottom = bottom;
        if(this.bottom < this.top)
        {  int temp = this.bottom;
           this.bottom = this.top;
           this.top = temp;
        }
    }
    /** this method sets the right coordinate of the rectangle, and makes sure it jives with the left coordinate
     @param right the new right coordinate of the rectangle
     */
    public void setRight(int right)
    {   this.right = right;
        if(this.right < this.left)
        {   int temp = this.right;
            this.right = this.left;
            this.left = temp;
        }
    }
    /** this is a debugging method for determining if the stupid rectangle is working properly
     @return String a string that can be used for debugging
     */
    public String report()
    {   return "left: " + this.left + " right: " + this.right + "top: " + this.top + " bottom: " + this.bottom;
    }
}
