package project.model;
import java.awt.*;
import becker.io.TextInput;
import becker.io.TextOutput;
public class Square extends Object
{    private SquareContents contents;

     private boolean isBesideWater;
     private boolean hasMulch;
     private boolean hasTopsoil;

     /** this constructor just starts the square of extremely basically, with empty squarecontents
     */
     public Square()
     {  super();
        this.contents = new SquareContents();
     }
     /** this is a slightly different constructor that starts the square of with an uneditable plant square
      @param isPlant this is just a ploy to alter the constructor for this constructor
     */
     public Square(boolean isPlant)
     {  super();
        this.contents = new SquareContents(isPlant);
     }
     /** this constructor loads from a textinput, uses the squarelist to figure out the contents of each square
      @param in - this textinput is initialized to a file compatible with this program
      @param squarelist - this is a list of all the possible squares, initialized from plantlist.txt
      */
     public Square(TextInput in, SquareList squareList)
     {  super();
        this.contents = squareList.searchCode(in.readToken());
     }
     /** this loads all the mulch in from a textinput
      @param in - the textInput object initalized to read in from a compatible file, advanced to the mulch section
      */
     public void loadMulch(TextInput in)
     {  String stype = in.readToken(); // reads in the current token in the row
        if(stype.trim().equalsIgnoreCase("T"))  // if it has topsoil
        {  this.hasTopsoil  = true;
           this.hasMulch = false;
        }
        else if(stype.trim().equalsIgnoreCase("B")) // if it has both
        {  this.hasTopsoil = true;
           this.hasMulch = true;
        }
        else if(stype.trim().equalsIgnoreCase("M")) // if it has mulch only
        {  this.hasTopsoil = false;
           this.hasMulch = true;
        }
        else if(stype.trim().equalsIgnoreCase("N")) // if it has neither
        {  this.hasTopsoil = false;
           this.hasMulch = false;
        }
        else
        {  throw new Error("faulty input"); // if one of them is wrong
        }
     }
     /** this spews out the plant/building code of whatever contents this square has
      @param out the textoutput initialized to whatever file we're saving to
      */
     public void save(TextOutput out)
     {   out.print(this.contents.getText ()+ " ");
     }
     /** this returns the display characters of this square's contents
      @return the display characters of this square's contents
      */
     public String getChars()
     {   return this.contents.getText();
     }
     /** this saves the mulch to an initialized textoutput file
      @param out the textoutput initialized to a file to save to
      */
     public void saveMulch(TextOutput out)
     {   if(this.hasMulch && this.hasTopsoil)
         {   out.print("B ");
         }
         else if(this.hasMulch && !this.hasTopsoil)
         {   out.print("M ");
         }
         else if(!this.hasMulch && this.hasTopsoil)
         {   out.print("T ");
         }
         else
         {   out.print("N ");
         }
     }
     /** this is a copy constructor used for the undo method in LandscapePlanner
      @param source - the source of the square, to copy from
      */
     public Square(Square source)
     {   this.contents = source.getContents();
         this.isBesideWater = source.getBesideWater ();
         this.hasMulch = source.getMulch();
         this.hasTopsoil = source.getTopsoil();
     }
     /** this returns the color of the contents of this square
      @return the color of the contents of this square
      */
     public Color getColor()
     {   return this.contents.getColor();
     }

     /** this puts a squareContents object into this square's contents object
      @param what we're planting/adding in this square
      */
     public void addContents(SquareContents whatToPut)
     {   this.contents = whatToPut;
     }
     /** this returns whether or not this is an empty square
      @return whether or not this square is blank
      */
     public boolean isBlank()
     {   return (this.contents.getType() == Constants.EMPTY_SQUARE);
     }
     /** this returns the contents of this square
      @return the contents of this square
      */
     public SquareContents getContents()
     {   return this.contents;
     }
     /** this sets whether or not this square is beside water
      @param ibw whether or not this square [I]s [B]eside [W]ater
      */
     public void setBesideWater(boolean ibw)
     {   this.isBesideWater = ibw;

     }
     /** this sets whether or not this square has mulch in it
      @param m whether or not this square has mulch
      */
     public void setMulch(boolean m)
     {  this.hasMulch = m;
        Debug.O("setting mulch to "+ m);
     }
     /** this sets whether or not this square has topsoil
      @param ts whether or not this square has topsoil
      */
     public void setTopsoil(boolean ts)
     {  this.hasTopsoil = ts;
     }
     /** this returns if this square is beside water
      @return if this square is beside water
      */
     public boolean getBesideWater()
     {   return this.isBesideWater;
     }
     /** this returns if this square has mulch in it
      @return whether or not this square has mulch
      */
     public boolean getMulch()
     {  return hasMulch;
     }
     /** this returns if this square has topsoil in it
      @return if this square has topsoil in it
      */
     public boolean getTopsoil()
     {  return hasTopsoil;
     }
     /** this method calculates and returns the environmental rating of this particular square
      @return the environmental cost of this square
      */
     public int getEnvironCost()
     {
        OrganicContents temp; // to hold the organic contents object of this square
        if(this.contents instanceof OrganicContents) // if the contents is actually organic, cast it
        {   temp = (OrganicContents) this.contents;
        }
        else
        {  return 0;
        }

        int total = 0;
        int waterPoints = calcWater(temp); // calculates how many points will get docked for this things water requirements
        int Co2Points = temp.getAir() + 1; // the air rating is 0 = low to 3 = very high, so adding one gets the proper amount of points to add
        total += waterPoints; // adds them on
        total += Co2Points;

        if(temp.getBlooms()) // if it blooms, add some
        {  total++;
        }
        if(temp.getDecid()) // if it's deciduous, add some
        {  total++;
        }
        total += temp.getEnviroBonus(); // add the environmental bonus from plantlist.txt

        return total;
     }
     /** this functionr returns the number of points to be deducted from the environmental score for water requirements
      @param temp the organiccontents object that this square contains
      @return and int representing how man points to be subtracted
      */
     private int calcWater(OrganicContents temp)
     {  int waterReq = convWaterReq(temp.getWaterReq()); // converts the LOW/MED/HI/VHI waterReq from temp into a centimetre value


        if(this.hasMulch) // if we have mulch, subtract MULCH_MOD
        {   waterReq -= Constants.MULCH_MOD;

        }
        if(this.isBesideWater) // if we are beside water, set water req to low
        {   waterReq = Constants.LOW_WATER_REQ;
        }

        int waterOver = waterReq - Constants.RAINFALL; // subtract rainfall from water requirements


        int waterPoints = 0;
        if(waterOver > 0) // if we still need more water, then
        {   waterPoints = (int) (-waterOver / 20); // waterpoints are negative, and they are excess cm/20
        }
        return waterPoints;
     }
     /** this helper method converts from the plantlist.txt water number into a centimetre value
      @return the centimetres of water the corresponds to the low/med/hi/vhi constants
      @param the waternum from plantlist.txt
      */
     private int convWaterReq(int waterNum)
     {   if(waterNum == 0)
         {   return Constants.LOW_WATER_REQ; // 30cm
         }
         else if(waterNum == 1)
         {   return Constants.MED_WATER_REQ; // 70cm
         }
         else if(waterNum ==2)
         {   return Constants.HI_WATER_REQ; // 110cm
         }
         else if(waterNum ==3)
         {   return Constants.VHI_WATER_REQ; // 150cm
         }
         else
         {   return 0;
         }
     }
     /** this returns this square's installation cost
      @param sl helps to determine if this is grass, which is the only plant that doesn't require a $2 mulch installation fee
      @return the installation cost for this square
      */
     public int getInstallCost(SquareList sl)
     {    int cost = 0;
          if(this.contents instanceof OrganicContents)
          {
             if(this.contents.getIsTree()) // if we are a tree, it is $25
             {   cost += 25;
             }
             else // if we are a flower, it is more expensive to install
             {   cost += 45;
             }

             if(this.hasTopsoil) // if we have topsoil costs a bit more
             {   cost += 5;
             }

             if(this.hasMulch) // if we have mulch, then it costs more
             {   cost += 5;
                 if(!this.contents.equals(sl.searchCode("GS"))) // if we aren't grass, mulch costs stuff to install
                 {   cost += 2;
                 }
             }
             cost += this.contents.getCost(); // add the purchase price of the plant
          }
          else // if we aren't organic...
          {   if(this.hasTopsoil) // we can still have topsoil and mulch if we aren't organic
              {   cost += 5;
              }
              if(this.hasMulch)
              {   cost += 5;
              }
          }
          return cost;
     }
     /** this function returns the maintanance cost of the square,
      @param sl the squarelist to help with determining the maintanance cost
      */
     public int getMaintainCost(SquareList sl)
     {    int cost = 0;
          cost += Constants.COST_PER_WATER_CM * this.calcExcessWater();
          return cost;
     }
     /** this function calculates the excess water in centimetres.  It is used in the maintanance cost calculations
      @return the excess water
      */
     private int calcExcessWater()
     {  OrganicContents temp;
        if(this.contents instanceof OrganicContents)
        {   temp = (OrganicContents)this.contents;
        }
        else
        {   return 0;
        }

        int waterReq = convWaterReq(temp.getWaterReq());

        if(this.hasMulch)
        {   waterReq -= Constants.MULCH_MOD;
        }
        if(this.isBesideWater)
        {   waterReq = Constants.LOW_WATER_REQ;
            Debug.O("it is beside water.  waterReq = " + waterReq);
        }

        int waterOver = waterReq - Constants.RAINFALL;
        return waterOver;
     }

     /** returns if this square's contents has a light problem, so the views can fix it
      @param hoursOfSun - the contents need to know how many hours of sun this square gets
      @return a true or false value representing if this square is having a lighting problem
     */
     public boolean lightProblem(int hoursOfSun)
     {  return this.contents.lightProblem(hoursOfSun);
     }
}
