package project.model;
import becker.io.*;
import java.awt.Color;

public class SquareList extends Object
{
    public static final int NUM_NON_PLANTS = 6;
    public static final int NUM_INORGANIC = 4;
    public static final int PLANT_END = NUM_NON_PLANTS - NUM_INORGANIC;

    SquareContents[] seeds;
    /** this initializes the seeds array from the plantlist.txt file usually.
     @param filename - where is the plantlist file?
     */
    public SquareList(String filename)
    { TextInput in = new TextInput(filename);
      this.seeds = new SquareContents[in.readInt()+NUM_NON_PLANTS]; // initializes the squarecontents array.  the first int in the file represents how many plants there are
      in.readLine(); // reads to the next line
      seeds[Constants.NUM_BUILDING] = new InorganicContents(Constants.BUILDING);
      seeds[Constants.NUM_CONCRETE] = new InorganicContents(Constants.PATHWAY); // these four lines start the first 4 indices to be the 4 basic inorganic types
      seeds[Constants.NUM_WATER] = new InorganicContents(Constants.WATER);
      seeds[Constants.NUM_SOIL] = new InorganicContents(Constants.SOIL);
      for(int x=NUM_INORGANIC;x<this.seeds.length-PLANT_END;x++)
      { this.seeds[x] = new OrganicContents(in); // initializes the rest from the plantlist file
      }
      int r = this.seeds.length-1; // this initializes the all-important empty square
      Constants.EMPTY_SQUARE = r; // ok, it's not really a constants.  ITS A CONSTANT IN DISGUISE
      this.seeds[r] = new SquareContents(); // starts up an empty square

      int z = this.seeds.length-2; // initializes another important square, the uneditable
      Constants.UNEDITABLE_SQUARE = z; // constant in disguise
      this.seeds[z] = new SquareContents(true); // starts up the uneditable square

    }
    /** this function puts together an array of all the organic plants in the array
     @return a string array of all the plants
     */
    public String[] compileArray() // puts together a string array of all the plants in the array
    {
        String[] myArray = new String[seeds.length-NUM_NON_PLANTS];
        int curIndex = 0;
        for(int x = NUM_INORGANIC;x < this.seeds.length - PLANT_END;x++)
        {   myArray[curIndex] = new String("Add " + seeds[x].getName());
            curIndex++;
        }
        return myArray;
    }
    /** Search the square list for a specific square name
    @param The name of the square
    @return the array index of the square that was found
    */
    public int search(String squareName)
    { for(int x = 0; x<seeds.length;x++)
      { if (seeds[x].getName().equalsIgnoreCase(squareName))
        { return x;
        }
        else
        {  Debug.O("String not found - |" + squareName + "| ... |" +seeds[x].getText()+ "|");
        }
      }
      return -1;
    }
    /** this returns the squareContents of a square that has a matching code with the parameter
     @return a squareContents that matches the code given
     @param code the display code of the square contents, like "GS" for grass
     */
    public SquareContents searchCode(String code)
    {  for(int x = 0;x < seeds.length;x++)
       {  if(seeds[x].getText().equalsIgnoreCase(code))
          { return seeds[x];
          }
          else
          {  Debug.O("Code not found - |" + code + "|...|" + seeds[x].getText() + "|");
          }
       }
       return null;
    }
    /** this returns a squarecontents out of the array based on the index given
     @return the squarecontents at seeds[squareNum]
     @param squareNum the array index to return
     */
    public SquareContents getSquare(int squareNum)
    { return seeds[squareNum];
    }

}
