package project.view;
import project.model.*;
import javax.swing.*;
import java.awt.event.*;

/** This Class adds a JCombo box for all the given specified options
    ie. Add topsoil, remove mulch etc.
    @author Art Hare, Waseem Quraishi and Kelly Leung
*/
public class AddRemCBOBox extends JComboBox
{
    private LandscapePlanner model;   //a reference to the model

    /** The constructor for this class
        @param actions the list of options in the JCombo Box
        @param model a reference to the model */
    public AddRemCBOBox(String[] actions, LandscapePlanner model)
    { super(actions);
      this.model = model;
      this.registerListeners();
    }

    /** This changes the list of options in this drop menu
        @param actions the given list of options */
    public void changeList(String[] actions)
    {  this.removeAllItems();
       for(int x= 0;x < actions.length;x++)
       {  this.addItem(actions[x]);
       }
    }

    /** An inner class that is used when registering an action listener */
    private class myAL implements ActionListener
    {
    /** This method calls parseText when a user selects one of the options from the menu)
        @param evt the ActionEvent */
      public void actionPerformed(ActionEvent evt)
      { parseText();
      }
    }

    /** Parses the text of the option selected from the drop menu */
    private void parseText()
    { if(this.getSelectedItem() == null)
      {   return;
      }
      if(this.getSelectedItem().toString().equalsIgnoreCase("add normal"))
      { this.model.setAction(Constants.ACTION_PLANT_NORMAL);
      }
      else if(this.getSelectedItem().toString().equalsIgnoreCase("add polygon"))
      { this.model.setAction(Constants.ACTION_PLANT_POLYGON);
      }
      else if(this.getSelectedItem().toString().equalsIgnoreCase("remove"))
      { this.model.setAction(Constants.ACTION_REMOVE);
      }
      else if(this.getSelectedItem().toString().equalsIgnoreCase("query"))
      { this.model.setAction(Constants.ACTION_QUERY);
      }
      else if(this.getSelectedItem().toString().equalsIgnoreCase("add mulch"))
      { this.model.setAction(Constants.ACTION_MULCH);
      }
      else if(this.getSelectedItem().toString().equalsIgnoreCase("remove mulch"))
      { this.model.setAction(Constants.ACTION_REM_MULCH);
      }
      else if(this.getSelectedItem().toString().equalsIgnoreCase("add topsoil"))
      { this.model.setAction(Constants.ACTION_TOPSOIL);
      }
      else if(this.getSelectedItem().toString().equalsIgnoreCase("remove topsoil"))
      { this.model.setAction(Constants.ACTION_REM_TOPSOIL);
      }
    }

    /** This registers the listener for this JCombo Box */
    private void registerListeners()
    { this.addActionListener(new myAL());
    }
}