package project.view;

import project.model.*;
import becker.io.TextInput;
import becker.util.IView;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.GridLayout;

/** A view of the planner which allow the user to enter commands.
 @author Art Hare, Waseem Quraishi and Kelly Leung */
public class CommandView extends JPanel implements IView
{
   /** GUI widgets needed. */
   private JComboBox actions;   //a drop menu of possible plants to add in the plant layer
   private JComboBox changeLayer;   //a drop menu to select which layer to view
   private JButton load;    //button to load files
   private JButton save;    //button to save files
   private JButton undo;    //button that undoes last action up to 5 times
   private JSlider shadowSlider;  //slider that shows the
   /** The Landscape Design Planner application logic. */
   private LandscapePlanner model;  //reference to the model
   private boolean removing;  //if the selected item from actions menu is remove
   private AddRemCBOBox addRemoveBox;   //Drop down menu of possible actions



   private boolean msgBoxDone = false;
   /** Construct the view of the planner which will provide user feedback.
    @param app the planner application logic
    */
   public CommandView(LandscapePlanner app)
   {  this.model = app;
      this.layoutView();
      this.registerListeners();
      this.model.addView(this);
   } // FeedbackView

   /** Construct and layout the components of the view. */
   private void layoutView()
   {
      this.actions = new JComboBox(this.model.getItemList());
      //this.updateCombo();
      this.shadowSlider = new JSlider(1,14,7);
      this.changeLayer = new JComboBox(this.model.getLayerList());
      this.changeLayer.setEnabled(this.model.getFull());
      this.setLayout(new GridLayout(7,1));
      this.addRemoveBox = new AddRemCBOBox(this.model.getActionList(), this.model);
      this.save = new JButton("Save");
      this.load = new JButton("Load");
      this.undo = new JButton("Undo");

      this.add(this.shadowSlider);
      this.add(this.load);
      this.add(this.save);
      this.add(this.actions);
      this.add(this.addRemoveBox);
      this.add(this.changeLayer);
      this.add(undo);
   } // layoutView
   /*
   private void updateCombo()
   {  this.actions.removeAllItems();
      String[] actions = this.model.getActionList();
      System.out.println(actions.length);
      for(int x = 0;x < actions.length;x++)
      {   this.actions.addItem(actions[x]);
      }
   } */
   /** Create and attach listeners to appropriate components of the view. */

   /** This sets the hour of the day given the appropriate hour from the shadow
       slider */
  private void processSlider()
   {   this.model.setDisplayHour(this.shadowSlider.getValue());
   }

   /** An inner class that registers a ChangeListener whenever the slider has
       been moved */
   private class ShadowSliderListener implements ChangeListener
   {
       /** This method calls processSlider to process the new information from
           the slider */
       public void stateChanged(ChangeEvent evt)
       {   processSlider();
       }
   }

   /** This registers all the various listeners needed for the GUI components */
   private void registerListeners()
   { this.actions.addActionListener(new UpdateActionListener());
     this.changeLayer.addActionListener (new ChangeLayerListener());
     this.save.addActionListener(new saveActionListener());
     this.load.addActionListener(new loadActionListener());
     this.undo.addActionListener(new undoListener());
     this.shadowSlider.addChangeListener (new ShadowSliderListener());
   } // registerListeners

   /** Refresh the view with up-to-date information from the application logic. */
   public void updateView()
   {  this.changeLayer.setEnabled(this.model.getFull());
      this.undo.setEnabled(this.model.undoAble());
      this.shadowSlider.show(this.model.getShowingShadow());
   } // updateView]

   /** An inner class that is used when registering an ActionListener for the
       Undo button */
   private class undoListener implements ActionListener
   {
      /** This method undoes the last action performed in the planner */
      public void actionPerformed(ActionEvent evt)
      {   model.undo();
          undo.setEnabled(model.undoAble());
      }
   }

   /** An inner class that is used when registering an ActionListener for the
       ChangeLayers button */
   private class ChangeLayerListener implements ActionListener
   {
      /** This switches the layers and updates GUI buttons/components when
          button is clicked */
      public void actionPerformed(ActionEvent evt)
      {  model.switchLayer(changeLayer.getSelectedItem ().toString());
         updateCombo();
      }
   }

   /** Updates the JCombo box to the appropriate list of options when layers
       have changed */
   private void updateCombo()
   {  this.removing = true;
      this.actions.removeAllItems();
      System.out.println("Cleared out combo box");
      String[] actionList = this.model.getItemList();
      System.out.println("length of array: " + actionList.length);
      for(int x = 0;x < actionList.length;x++)
      {  this.actions.addItem(actionList[x] );
         System.out.println("adding to combo box");
      }
      this.addRemoveBox.changeList(this.model.getActionList());
   }

   /** An inner class that is used when registering an ActionListener when
       the items of the actions drop-menu is selected */
  private class UpdateActionListener implements ActionListener
	{
    /* Inform the model that the user has chosen a new value
       with which to fill the grid squares.
       @param evt the ActionEvent
    */
    public void actionPerformed(ActionEvent evt)
		{
      if(!removing)
      {   String updateActionChosen = (String)((JComboBox)evt.getSource()).getSelectedItem();
          model.setActionValue(updateActionChosen);
      }
      else
      {   removing = false;
      }
		}
  }

  /** An inner class that is used when registering an ActionListener for the
      load button */
  private class loadActionListener implements ActionListener
  {
     /** This method calls loadFile when user clicks the load button */
     public void actionPerformed(ActionEvent evt)
     {  loadFile();
     }
  }

  /** An inner class that is used when registering an ActionListener for the
      Save Button */
  private class saveActionListener implements ActionListener
  {
    /** This methods calls saveFile when user clicks the save button */
    public void actionPerformed(ActionEvent evt)
    {  saveFile();
    }
  }

  /** Creates a new temporary loading file box that allows user to enter a filename */
  private void loadFile()
  {   LoadMsgBox myMB = new LoadMsgBox(this);
  }
  /** Calls the SimpleMain class to load the given filename */
  public void load(String file)
  {   SimpleMain.loadFile(file);;
  }
  /** Creates a new temporary saving file box that allows user to enter a filename */
  private void saveFile()
  {   SaveMsgBox myMB = new SaveMsgBox(this);
  }
  /** Calls the model to save the data in an appropriate format in the given filename */
  public void save(String file)
  {   this.model.save(file);
  }



} // FeedbackView