package project.view;


import project.model.*;

import becker.util.IView;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
/** A view of the planner which will provide feedback to the user.
 @author Art Hare, Waseem Quraishi and Kelly Leung */
public class FeedbackView extends JPanel implements IView
{

   /** Constants for the dimensions of the area. */
   public static final int AREA_WIDTH = 150;
   private static final int NUM_CHARS = 15;
   private static final int NUM_ROWS = 20;

   /** GUI widgets needed. */
   private JTextArea userInfo = new JTextArea(NUM_ROWS, NUM_CHARS);   //Provides user with current activity
   private JTextArea envData = new JTextArea("Environmental Data not available",4,10);    //Shows info of the environmental impact of landscape design
   private JTextArea costData = new JTextArea("Cost Data not available",4,10);    //Shows intial and maintenance cost of landscape design
   /** The Landscape Design Planner application logic. */
   private LandscapePlanner model;  //a reference to the model

   /** Construct the view of the planner which will provide user feedback.
    @param app the planner application logic
    */
   public FeedbackView(LandscapePlanner app)
   {  this.model = app;
      this.layoutView();
      this.registerListeners();
      this.model.addView(this);
   } // FeedbackView

   /** Construct and layout the components of the view. */
   private void layoutView()
   {  JScrollPane areaScrollPane = new JScrollPane(userInfo);
      areaScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
      this.userInfo.setLineWrap(true);
      this.setLayout(new GridLayout(5,1));
      this.add(areaScrollPane);
      this.add(this.envData);
      this.add(this.costData);
   } // layoutView

   /** Create and attach listeners to appropriate components of the view. */
   private void registerListeners()
   { // Nothing yet
   } // registerListeners

   /** Refresh the view with up-to-date information from the application logic. */
   public void updateView()
   {  this.userInfo.setText(this.model.getQuery());
      int install = this.model.getInstallCost();
      int maintain = this.model.getMaintainCost();

      this.costData.setText("Total Install Cost: $" + install + "\nTotal Maintenance Cost: $" + maintain + "\nTotal Cumulative Cost: " + (install + maintain));
   } // updateView
   
} // FeedbackView
