package project.view;


import project.model.*;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.BorderFactory;

/** A section of the 2-D grid layout area.
 @author Art Hare, Waseem Quraishi and Kelly Leung */
public class GridSection extends JLabel
{
	/* NOTE:  You may be tempted to put in many more instance variables
	 * about what kind of soil is here or if this square contains plants or
	 * ....  DON'T DO IT!  The view is NOT the place for that kind of
	 * information -- put it in the model. */

   /** Constants for the dimensions of a grid square. */
   public static final int SQUARE_W = 40;
   public static final int SQUARE_H = 40;

   /** The point the square appears on the grid. */
   private int col; //column
   private int row; //row

   private GridView myGV; //a reference to GridView
   private Square mySquare;   //a reference to the Square object of the grid
   /** Create a graphical representation of a grid square in the planner.
    @param x the x coordiate
    @param y the y coordinate
    @param value a reference to the Square object
    @param value the "graphical" representation of the data
    */
   public GridSection(int x, int y, Square value,GridView myGV)
   {  //super(value.getChars(), SwingConstants.CENTER);
   	  this.col = x;
   	  this.row = y;
      this.myGV = myGV;
      this.mySquare = value;
      this.setPreferredSize(new Dimension(SQUARE_W, SQUARE_H));
      this.setBorder(new EtchedBorder(EtchedBorder.RAISED));
      this.registerListeners();
      this.setOpaque(true);
      //this.setBackground(mySquare.getColor());

   } // GridSection

   /** An inner class that registers a MouseMotionListner */
   private class MouseMotionListenerz implements MouseMotionListener
   {

        public void mouseMoved(MouseEvent evt)
        {
             myGV.setCurrentSection(col,row);
        }
        public void mouseDragged(MouseEvent evt)
        {
        }
        
   }
   private class myML implements MouseListener
   {
        public void mousePressed(MouseEvent e)
        {
        }

        public void mouseReleased(MouseEvent e)
        {
        }
        public void mouseEntered(MouseEvent e)
        {
        }
        public void mouseExited(MouseEvent e)
        {
        }
        public void mouseClicked(MouseEvent e)
        {
        }
   }

   /** Registers a listner for the mouse to get information over where the is
       at */
   private void registerListeners()
   {
      this.addMouseMotionListener (new MouseMotionListenerz());
   }

   /** @return the xy-coordinate of the grid square. */
   public int getRow()
   {  return this.row;
   }

   /** @return the xy-coordinate of the grid square. */
   public int getCol()
   {  return this.col;
   } 
   
   /** Updates the "graphical representation of the square
       @param mySquare a reference to the Square object on the grid
       @param model a referece to the model */
   public void updateData(Square mySquare,LandscapePlanner model)
   {  this.setText(mySquare.getChars());
      this.setForeground (mySquare.getColor());
      this.drawBorder(mySquare,model);
      this.mySquare = mySquare;
      this.repaint();

   } // updateData

   /** This function alerts the user that a given square doesn't have light by coloring the border red
    @param mySquare the square the gridSection is based on
    @param model the landscape model
    */
   private void drawBorder(Square mySquare,LandscapePlanner model)
   {  if(mySquare.lightProblem(model.getSun(this.getCol(),this.getRow()))) // if the square has a light problem...
      {   this.setBorder(BorderFactory.createEtchedBorder(new Color(255,0,0),new Color(200,200,200))); // then colour it red
      }
      else
      {   this.setBorder(BorderFactory.createEtchedBorder());
      }
   }

   /** This function paints the blue, red, and brown squares on the label, to represent water, topsoil, and mulch respectively
    @param g this is a graphics component passed by java
    */
   public void paintComponent(Graphics g)
   {    super.paintComponent(g); // not a problem
        Graphics2D g2 = (Graphics2D)g; // must stay
        BasicStroke stroke = new BasicStroke(0.01F) ; // seemingly not a problem
        g2.setStroke(stroke);    // seemingly not a problem


        if(mySquare.getMulch())  // if there is mulch...
        {   g2.setColor(new Color(128,64,0)); // draw a brown square
            g2.fillRect(0,0,10,10);
        }

        if(mySquare.getBesideWater ()) // if there is water...
        {   g2.setColor(new Color(0,0,255));  // draw a blue square
            g2.fillRect(10,0,10,10);
        }


        if(mySquare.getTopsoil()) // if there is topsoil
        {   g2.setColor(new Color(255,0,0));  // draw a red square
            g2.fillRect(20,0,10,10);
        }
   }

} // GridSection
